<?php

namespace Drupal\cacheexclude\EventSubscriber;

use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\node\NodeInterface;

/**
 * Event subscriber for excluding pages from the cache.
 *
 * @package Drupal\cacheexclude.
 */
class CacheexcludeSubscriber implements EventSubscriberInterface {

  /**
   * Constructs a new CacheexcludeSubscriber.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Path\PathMatcherInterface $pathMatcher
   *   The path matcher.
   * @param \Drupal\Core\Path\CurrentPathStack $currentPath
   *   The current path stack.
   * @param \Drupal\path_alias\AliasManagerInterface $aliasManager
   *   The alias manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $routeMatch
   *   The route match.
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $killSwitch
   *   The page cache kill switch.
   */
  public function __construct(
    protected ConfigFactoryInterface $configFactory,
    protected PathMatcherInterface $pathMatcher,
    protected CurrentPathStack $currentPath,
    protected AliasManagerInterface $aliasManager,
    protected RouteMatchInterface $routeMatch,
    protected KillSwitch $killSwitch,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = [];
    $events[KernelEvents::REQUEST][] = ['disableCacheForPage'];

    return $events;
  }

  /**
   * Subscriber Callback for the event.
   */
  public function disableCacheForPage(): void {
    // Path checking routine.
    if ($this->checkPath() === TRUE) {
      // Disable page cache temporarily.
      $this->killSwitch->trigger();
      return;
    }

    // Check if current node type is one we want to exclude from the cache.
    $node = $this->routeMatch->getParameter('node');
    if ($node instanceof NodeInterface) {
      $node_type = $node->getType();
    }

    $config = $this->configFactory->get('cacheexclude.settings');
    $node_types = $config->get('cacheexclude_node_types');

    if (!empty($node_types)) {
      $node_types = array_filter($node_types);
      if (isset($node_type) && in_array($node_type, $node_types, TRUE)) {
        // Disable page cache temporarily.
        $this->killSwitch->trigger();
      }
    }
  }

  /**
   * Checks whether module configuration excludes the current path.
   *
   * @return bool
   *   TRUE if the path should not be cached.
   */
  private function checkPath(): bool {
    // Get cacheexclude page configuration.
    $config = $this->configFactory->get('cacheexclude.settings');
    // Only trim if config exists.
    $pages = !empty($config->get('cacheexclude_list')) ? trim($config->get('cacheexclude_list')) : NULL;

    // If the current page is one we want to exclude from the cache,
    // disable page cache temporarily.
    if (!is_null($pages)) {
      $current_path = $this->currentPath->getPath();
      $current_path_alias = $this->aliasManager->getAliasByPath($current_path);
      $path_matches = $this->pathMatcher->matchPath($current_path, $pages);
      $alias_path_matches = $this->pathMatcher->matchPath($current_path_alias, $pages);

      if ($path_matches || $alias_path_matches) {
        return TRUE;
      }

      $system404 = $this->configFactory->get('system.site')->get('page.404');

      if (isset($system404)) {
        if (is_array($pages) && in_array($current_path, $pages)) {
          return TRUE;
        }
        elseif ($system404 === $pages) {
          return TRUE;
        }
      }
    }

    return FALSE;
  }

}
